import { Parser } from "@siteimprove/alfa-parser";
import { Function as CSSFunction } from "../../syntax/index.js";
import { List } from "../collection/index.js";
import { Number } from "../numeric/index.js";
import { Function } from "./function.js";
const { map, either } = Parser;
/**
 * @public
 */
export class Scale extends Function {
    static of(x, y) {
        return new Scale(x.resolve(), y.resolve());
    }
    _x;
    _y;
    constructor(x, y) {
        super("scale", false);
        this._x = x;
        this._y = y;
    }
    get kind() {
        return "scale";
    }
    get x() {
        return this._x;
    }
    get y() {
        return this._y;
    }
    resolve() {
        return this;
    }
    equals(value) {
        return (value instanceof Scale &&
            value._x.equals(this._x) &&
            value._y.equals(this._y));
    }
    hash(hash) {
        hash.writeHashable(this._x).writeHashable(this._y);
    }
    toJSON() {
        return {
            ...super.toJSON(),
            x: this._x.toJSON(),
            y: this._y.toJSON(),
        };
    }
    toString() {
        if (this._x.value === this._y.value) {
            return `scale(${this._x})`;
        }
        if (this._y.value === 1) {
            return `scaleX(${this._x})`;
        }
        if (this._x.value === 1) {
            return `scaleY(${this._y})`;
        }
        return `scale(${this._x}, ${this._y})`;
    }
}
/**
 * @public
 */
(function (Scale) {
    function isScale(value) {
        return value instanceof Scale;
    }
    Scale.isScale = isScale;
    /**
     * {@link https://drafts.csswg.org/css-transforms/#funcdef-transform-scale}
     */
    const parseScale = map(CSSFunction.parse("scale", map(List.parseCommaSeparated(Number.parse, 1, 2), (list) => list.values)), ([_, [x, y]]) => Scale.of(x, y ?? x));
    /**
     * {@link https://drafts.csswg.org/css-transforms/#funcdef-transform-scalex}
     */
    const parseScaleX = map(CSSFunction.parse("scaleX", Number.parse), ([_, x]) => Scale.of(x, Number.of(1)));
    /**
     * {@link https://drafts.csswg.org/css-transforms/#funcdef-transform-scaley}
     */
    const parseScaleY = map(CSSFunction.parse("scaleY", Number.parse), ([_, y]) => Scale.of(Number.of(1), y));
    Scale.parse = either(parseScale, parseScaleX, parseScaleY);
})(Scale || (Scale = {}));
//# sourceMappingURL=scale.js.map